/****************************************************************/
// Funktion SIM7600E - Mobilfunk  TCP / SSL-Send  EE-Prom
/****************************************************************/
// 29.07.2020 SIM7600
// 07.08.2020 EE-Prom Checksumme
// 18.08.2020 Debug Ausgabe optimiert
// 03.12.2020 Sendezyklen konfigurierbar
// 08.01.2021 Debug Info
// 19.03.2021 String handling
// 03.05.2021 EE-Prom-only
// 25.05.2021 EE-Prom error handling
// 12.11.2021 Close Session optimized
// 28.11.2021 Debug Messages
// 28.03.2022 Pegelmessung
// 07.04.2023 modify Http calls, TCP new
// 17.04.2023 add signal level measurement
// 07.02.2024 Datenstring Signalpegel
// 08.02.2024 wait a sec before sending next data
// 14.07.2024 Mx-Strings
// 18.07.2024 Use EE-Prom 'round robin'
// 10.01.2025 add debugflush
// 12.01.2025 Send A=1 obsolete


#if (_LTE_TLS == 0)

/****************************************************************/
// Kommunikation SIM7600E - Mobilfunk
/****************************************************************/
#include "SIM7600_beelogger.h"

#define SERVER_TM_OUT 10000 // 10000 = 10 sec
/****************************************************************/


/****************************************************************/
// Parameter Datenversand via Http // diese nicht modifizieren
/****************************************************************/
const char Str_Pw[]      PROGMEM = {"PW="};
const char Str_Id[]      PROGMEM = {"&Z=2&ID="};
#if(Anzahl_Sensoren_Gewicht == 1)
const char Str_MData[] PROGMEM = { "&M1=" };  // Parameter Multi Data  Senden
#endif
#if(Anzahl_Sensoren_Gewicht == 2)
const char Str_MData[] PROGMEM = { "&M2=" };  // Parameter Multi Data  Senden
#endif
#if(Anzahl_Sensoren_Gewicht == 3)
const char Str_MData[] PROGMEM = { "&M3=" };  // Parameter Multi Data  Senden
#endif
#if(Anzahl_Sensoren_Gewicht == 4)
const char Str_MData[] PROGMEM = { "&M4=" };  // Parameter Multi Data  Senden
#endif
/****************************************************************/

/**
  @brief Utility functions
  stop_DFUe_device(): SIM7600E Reset aktiv oder Power inaktiv
  dfue_rxtx_off():  Serial TX/RX dfiniert setzen

  @param  none, globale Variablen
  @return  none
*/
void stop_DFUe_device() {
  digitalWrite(GSM_Power_Pin, LOW);    // SIM7600E-Modul "off" or Reset
  pinMode(GSM_Power_Pin, OUTPUT);
}

void dfue_rxtx_off() {
  digitalWrite (GSM_RX, LOW);
  digitalWrite (GSM_TX, LOW);
}


/**
  @brief Funktion SIM7600E Send_Data()
  initialisiere SIM7600E, vebinde ins Mobilfunknetz
  verbinde zum Server, lese Daten aus EE-Prom,
  übertrage Daten, Server-Quittung auswerten
  Fehlerbehandlung bei der Übertragung
  Setze aus Serverquittung:
  - Zeit in DS3231
  - Messintervall
  - Sendeintervall

  @param  count:: Anzahl Datensätze zu übertragen, globale Variablen
  @return  senden erfolgreich > 0
*/

uint8_t Send_Data(uint8_t count) {
  char parameter[64];
  uint8_t s_cnt = 0;            // send counter
  uint8_t additonal_Data = 0;

  uint8_t x, y;       // Datensatz zaehler

  uint8_t sum, test, first; // EE-Prom Checksumme
  char chk[2];

  uint16_t pos;
  uint8_t data_send;    // send status
  uint8_t retry;        // retry counter

  uint8_t sets_to_Send = _Send_Sets;     //

  CGPRS_SIM7600 dfue;

  debugprintlnF("LTEon");debugflush();

  digitalWrite(GSM_Power_Pin, HIGH);
  calldelay(4000); // Important, wait for SIM7600coming up

  if (dfue.init(6000)) {  // Init SIM7600 Modul

    debugprintlnF("Init!"); debugflush();

    strcpy_P(parameter, APN); debugprintln(parameter); debugflush();

    if (dfue.start(parameter, APN_Benutzer, APN_Passwort) == 0) { // Start dfue Connection
      //#################
      data_send = 0;    // send status
      retry = 0;        // retry counter
      do {  // retry loop

        do {  // Blockweise send loop
          data_send = 0;
          // always new connect; session alive not supported by server
          strcpy_P(parameter, serverName);
          if (dfue.Connect(parameter)) {  // Start TCP Connection    CIPOPEN
            dfue.sendCommand("AT");
            debugprintlnF("TCP"); debugflush();

            if (Service < 90.0) {      // if user action, force time_on
              if (report_info == 1) {  // pegelmessung
                calldelay(3000); // 28.03.2022

                int Signal = dfue.getSignalQuality();
                Service = (float)Signal;
                additonal_Data = 1;

                debugprint(Signal);
                debugprintlnF(" dBm");
                debugflush();

              }
            }

            if (dfue.prep_send()) {  // Open Server connection

              strcpy_P(parameter, beelogger_pfad); dfue.send(parameter);
              debugprintln(parameter); debugflush();

              strcpy_P(parameter, Str_Pw); dfue.send(parameter);
              strcpy_P(parameter, Passwort); dfue.send(parameter);

              // System ID
              strcpy_P(parameter, Str_Id); dfue.send(parameter);
              debugprint(parameter); debugflush();
              strcpy(parameter, ID_ID); dfue.send(parameter);
              calldelay(50);
              debugprintln(parameter); debugflush();

              //Daten
              y = 0; first = 1;
              do {
                // lese von AT24Cxx
                test = 0;
                do {
                  pos = data_start + (uint16_t)(s_cnt + y);
                  if (pos >= _EE_DATA_SETS) pos -= _EE_DATA_SETS;
                  pos *= _Daten_Satz_Len;
                  my_EEPROM.ReadStr(pos, DatenString, _Daten_Satz_Len - 2);
                  DatenString[(_Daten_Satz_Len - 2)] = 0;  // String ende erzwingen
                  sum = 0;
                  for (x = 0; x < strlen(DatenString); x++) { // EE_Prom Checksum test
                    sum += DatenString[x];
                  }
                  pos = pos + _Daten_Satz_Len - 1;
                  my_EEPROM.ReadStr(pos, chk, 1);
                  x = (uint8_t)chk[0];
                  test++;
                } while ((test < 2) && ( sum != x)); // zwei Leseversuche

                if (sum == x) {           // checksumme i.o.
                  if (first == 1) {       // einfügen Satztrenner
                    first = 0;
                    strcpy_P(parameter, Str_MData);
                    dfue.send(parameter);

                    debugprint(parameter); debugflush();
                  }
                  else                 {
                    dfue.send(",");
                  }
                  dfue.send(DatenString);
                  calldelay(50);
                  debugprintln(DatenString); debugflush();

                }
                else {                    // Datensatz auslassen
                  DatenString[0] = 0;
                  debugprintF("EE?"); debugprintln(pos); debugflush();
                }
                y++;

              } while ( (y < sets_to_Send) && ((s_cnt + y) < count) );


              if ( (s_cnt + y) == count) { // all done, Service ?
                if (additonal_Data) {
                  char service_data[16];
                  dtostrf(Service, 1, 1, service_data);
                  sprintf(DatenString, "&S=%s", service_data);
                  dfue.send(DatenString);
                  debugprintln(DatenString); debugflush();
                }
              }
              // ende Daten
              dfue.send(" HTTP/1.1\r\n"); // HTTP/1.1
              dfue.send("Host: "); strcpy_P(parameter, serverName); dfue.send(parameter); dfue.send ("\r\n");
              dfue.send("Connection: keep-alive\r\n");
              dfue.send("\r\n");
              calldelay(20);
              dfue.send(0x00); // Startkommando senden
              // http complete

              if (dfue.sendCommand(0, SERVER_TM_OUT, "ok *")) { // hat Server Anfrage quittiert ?
                data_send = 1; // success
                retry = 0;
                s_cnt += sets_to_Send;

                char *p_buf = strstr(dfue.buffer, "ok *");  // search string start
                uint8_t pos = 0;
                do {
                  p_buf --;    // search start of line
                  pos++;
                }
                while ( (!(*p_buf == '\n')) && (pos < 20));

                debugprintF("Q:"); debugprintln(p_buf); debugflush();

                // Format Quittung aus beelogger-log.php
                // echo "5ok *"  = 5 Minuten
                uint8_t n;
                if (char *p_bf = strchr(p_buf, 'T') ) {
                  n = atoi(p_bf + 1);  // Konvertiere str in int

                  char x = *(p_buf + 1);
                  if ((x > 0x2F) && (x < 0x3A)) { // first char a number?
                    long l_tm = atol(p_buf);
                    l_tm = l_tm - 946684800;  // EPOCH_TIME_OFFSET
                    rtc.setDateTime(l_tm);
                  }
                }
                else {
                  n = atoi(p_buf); // Konvertiere str in int
                }
                if (n <= 60) {
                  WeckIntervallMinuten = (uint8_t) n;
                }

                report_info = 0;
                if (strchr(p_buf, 'P') ) {
                  report_info = 1;
                }

                if (strchr(p_buf, 'A') ) {
                  send_cycle = SEND_CYC_A;
                }
                else if (strchr(p_buf, 'B') ) {
                  send_cycle = SEND_CYC_B;
                }
                else if (strchr(p_buf, 'C') ) {
                  send_cycle = SEND_CYC_C;
                }
                else if (strchr(p_buf, 'D') ) {
                  send_cycle = SEND_CYC_D;
                }
                calldelay(1000);
              }  // received ok *
            }  // Open / Connect
            debugprintlnF("cls:");debugflush();
            dfue.disConnect();        // stop Network  CIPCLOSE
          } // Connect
          if (s_cnt < count) delay (2000);
        }
        while ((s_cnt < count) && (data_send == 1)); // while my_counter
        //######################
        if (data_send == 0) {  // failed, stop and retry
          debugprintlnF("Retry"); debugflush();
          calldelay(2000);
          retry ++;
          if (sets_to_Send > 1) sets_to_Send --;
        }
      }
      while ((retry < 3) && (data_send == 0)); // retry TCP Connect

      dfue.stop();        // stop Network NETCLOSE

    } // start APN
  } // init

#if myDEBUG
  if (s_cnt > count) s_cnt = count;
#endif
  debugprintF("send "); debugprintln(s_cnt); debugflush();

  dfue.shutdown();    // shutdown dfue
  calldelay(50);
  dfue_rxtx_off();
  digitalWrite(GSM_Power_Pin, LOW);
  return (s_cnt);
}
#endif
